import { Body, Config, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { AdminService } from '../service/admin.service';
import { CreateDTO_admin, CreateVO_admin, FetchVO_admin, LoginDTO, LoginVO, QueryDTO_admin, QueryVO_admin, ResetPasswordDTO, UpdateDTO_admin, UpdateVO_admin } from '../interface/admin.interface';
import { ApiOperation, ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { JwtService } from '@midwayjs/jwt';
import get = require('lodash/get');
import lowerCase = require('lodash/lowerCase');
import { RegisterCanNotIncludes } from '../util/util';
@ApiTags(['管理员账号'])
@Controller('/admin')
export class AdminController extends BaseController {
  @Inject()
  service: AdminService;

  @Inject()
  jwtService: JwtService;
  @Config('jwt.secret')
  jwtSecret;
  @Config('jwt.expiresIn')
  jwtExpiresIn;

  @Post('/login')
  @Validate()
  async login(@Body() body: LoginDTO) {
    const data = await this.service.findUserToLogin(body);
    let vo = new LoginVO(data);
    vo = JSON.parse(JSON.stringify(vo));
    const token = await this.jwtService.sign(vo, this.jwtSecret, { expiresIn: this.jwtExpiresIn });
    return token;
  }

  @Post('/rp')
  @ApiOperation({ description: '修改密码' })
  async resetPassword(@Body() body: ResetPasswordDTO) {
    // 获取用户token中的登录信息
    const user_id = get(this.ctx, 'user._id');
    if (!user_id) throw new ServiceError('未找到用户信息', FrameworkErrorEnum.NOT_LOGIN);
    // 查询用户
    const user = await this.service.fetch(user_id);
    if (!user) throw new ServiceError('未找到用户', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 修改密码
    await this.service.updateOne(user_id, body);
    // 返回结果
    return 'ok';
  }

  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_admin })
  async create(@Body() data: CreateDTO_admin) {
    // 检查是否含有不允许字符
    const haveInvalid = RegisterCanNotIncludes.find(f => lowerCase(data.account).includes(f));
    if (haveInvalid) throw new ServiceError(`不允许包含特殊字符: ${RegisterCanNotIncludes.join('; ')}`);
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QueryVO_admin })
  async query(@Query() filter: QueryDTO_admin, @Query('skip') skip: number, @Query('limit') limit: number) {
    filter.is_super = false;
    const data = await this.service.query(filter, { skip, limit });
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_admin })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FetchVO_admin(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_admin })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_admin) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
