import { Body, Config, Controller, Del, Get, Inject, InjectClient, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { UnitService } from '../service/unit.service';
import {
  CreateDTO_unit,
  CreateVO_unit,
  EmailResetPasswordDTO,
  FetchVO_unit,
  LoginDTO,
  LoginVO,
  QueryDTO_unit,
  QueryVO_unit,
  ResetPasswordDTO,
  UnitForgetEmailDTO,
  UpdateDTO_unit,
  UpdateVO_unit,
} from '../interface/unit.interface';
import { ApiOperation, ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { JwtService } from '@midwayjs/jwt';
import { RedisService } from '@midwayjs/redis';
import { HttpService, HttpServiceFactory } from '@midwayjs/axios';
import { randomStr, RegisterCanNotIncludes } from '../util/util';
import get = require('lodash/get');
import lowerCase = require('lodash/lowerCase');
import { UserUtilService } from '../service/util/user.util';
@ApiTags(['单位账号'])
@Controller('/unit')
export class UnitController extends BaseController {
  @Inject()
  service: UnitService;
  @Inject()
  jwtService: JwtService;

  @Inject()
  userUtil: UserUtilService;

  @Config('jwt.secret')
  jwtSecret;
  @Config('jwt.expiresIn')
  jwtExpiresIn;

  @Inject()
  redisService: RedisService;

  @Config('redis_timeout')
  redisTimeout;

  @InjectClient(HttpServiceFactory, 'email')
  emailAxios: HttpService;
  @Config('emailConfig')
  emailConfig;

  emailKey = 'unitForgetEmail:';

  @Post('/login')
  @Validate()
  async login(@Body() body: LoginDTO) {
    const data = await this.service.findUserToLogin(body);
    let vo = new LoginVO(data);
    vo = JSON.parse(JSON.stringify(vo));
    const token = await this.jwtService.sign(vo, this.jwtSecret, { expiresIn: this.jwtExpiresIn });
    return token;
  }

  @Post('/tfe')
  @ApiOperation({ description: '发送重置密码 验证码邮件' })
  async toForgetEmail(@Body() body: UnitForgetEmailDTO) {
    const rstr = randomStr();
    const data = await this.service.findOne(body);
    if (!data) throw new ServiceError('未找到该账号的用户', FrameworkErrorEnum.NOT_FOUND_DATA);
    const key = `${this.emailKey}${rstr}`;
    await this.redisService.set(key, JSON.stringify(body), 'EX', this.redisTimeout);
    // sendEmail
    const uri = '/sendEmail';
    await this.emailAxios.post(uri, { config: this.emailConfig, template: ['jcyjdtglpt', 'toResetPassword'], receiver: get(data, 'email'), params: { code: rstr } });
    return 'ok';
  }

  @Post('/erp')
  @ApiOperation({ description: '忘记密码:带验证码重置账号的密码' })
  async emailResetPassword(@Body() body: EmailResetPasswordDTO) {
    const key = `${this.emailKey}${body.code}`;
    let rres = await this.redisService.get(key);
    if (!rres) throw new ServiceError('验证码已过期,请重新操作', FrameworkErrorEnum.SERVICE_FAULT);
    rres = JSON.parse(rres);
    if (get(rres, 'account') !== body.account) throw new ServiceError('该验证不是当前账号的验证码,拒绝操作', FrameworkErrorEnum.SERVICE_FAULT);
    const res = await this.service.updatePassword(body);
    return res;
  }
  @Post('/rp')
  @ApiOperation({ description: '修改密码' })
  async resetPassword(@Body() body: ResetPasswordDTO) {
    // 获取用户token中的登录信息
    const user_id = get(this.ctx, 'user._id');
    if (!user_id) throw new ServiceError('未找到用户信息', FrameworkErrorEnum.NOT_LOGIN);
    // 查询用户
    const user = await this.service.fetch(user_id);
    if (!user) throw new ServiceError('未找到用户', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 修改密码
    await this.service.updateOne(user_id, body);
    // 返回结果
    return 'ok';
  }

  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_unit })
  async create(@Body() data: CreateDTO_unit) {
    // 检查是否含有不允许字符
    const haveInvalid = RegisterCanNotIncludes.find(f => lowerCase(data.account).includes(f));
    if (haveInvalid) throw new ServiceError(`不允许包含特殊字符: ${RegisterCanNotIncludes.join('; ')}`);
    await this.userUtil.checkCardAndAccount(data);
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QueryVO_unit })
  async query(@Query() filter: QueryDTO_unit, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_unit })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FetchVO_unit(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_unit })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_unit) {
    await this.userUtil.checkUpdateCardAndAccount(id, body);
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
