import { Body, Config, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { UserService } from '../service/user.service';
import {
  CreateDTO_user,
  CreateVO_user,
  EmailResetPasswordDTO,
  FetchVO_user,
  LoginDTO,
  LoginVO,
  QueryDTO_user,
  QueryVO_user,
  ResetPasswordDTO,
  UpdateDTO_user,
  UpdateVO_user,
  UserForgetEmailDTO,
} from '../interface/user.interface';
import { ApiOperation, ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { JwtService } from '@midwayjs/jwt';
import { RedisService } from '@midwayjs/redis';
import { randomStr, RegisterCanNotIncludes } from '../util/util';
import { HttpServiceFactory, HttpService } from '@midwayjs/axios';
import { InjectClient } from '@midwayjs/core';
import get = require('lodash/get');
import lowerCase = require('lodash/lowerCase');
import { RoleService } from '../service/role.service';
import { UserUtilService } from '../service/util/user.util';

@ApiTags(['个人账号'])
@Controller('/user')
export class UserController extends BaseController {
  @Inject()
  service: UserService;

  @Inject()
  jwtService: JwtService;

  @Inject()
  roleService: RoleService;

  @Inject()
  userUtil: UserUtilService;

  @Config('jwt.secret')
  jwtSecret;
  @Config('jwt.expiresIn')
  jwtExpiresIn;

  @Inject()
  redisService: RedisService;

  @Config('redis_timeout')
  redisTimeout;

  @InjectClient(HttpServiceFactory, 'email')
  emailAxios: HttpService;
  @Config('emailConfig')
  emailConfig;

  emailKey = 'userForgetEmail:';

  @Post('/util')
  async util() {
    await this.service.test();
  }

  @Post('/login')
  @Validate()
  async login(@Body() body: LoginDTO) {
    const data = await this.service.findUserToLogin(body);
    let vo = new LoginVO(data);
    vo = JSON.parse(JSON.stringify(vo));
    // 找各个项目的关系
    await this.service.getModulesRelation(vo);
    // 判断可进入的模块
    await this.service.whitchModuleCanUse(vo);
    const token = await this.jwtService.sign(vo, this.jwtSecret, { expiresIn: this.jwtExpiresIn });
    return token;
  }

  @Post('/tfe')
  @ApiOperation({ description: '发送重置密码 验证码邮件' })
  async toForgetEmail(@Body() body: UserForgetEmailDTO) {
    const rstr = randomStr();
    const data = await this.service.findOne(body);
    if (!data) throw new ServiceError('未找到该账号的用户', FrameworkErrorEnum.NOT_FOUND_DATA);
    const key = `${this.emailKey}${rstr}`;
    await this.redisService.set(key, JSON.stringify(body), 'EX', this.redisTimeout);
    // sendEmail
    const uri = '/sendEmail';
    await this.emailAxios.post(uri, { config: this.emailConfig, template: ['jcyjdtglpt', 'toResetPassword'], receiver: get(data, 'email'), params: { code: rstr } });
    return 'ok';
  }

  @Post('/erp')
  @ApiOperation({ description: '忘记密码:带验证码重置账号的密码' })
  async emailResetPassword(@Body() body: EmailResetPasswordDTO) {
    const key = `${this.emailKey}${body.code}`;
    let rres = await this.redisService.get(key);
    if (!rres) throw new ServiceError('验证码已过期,请重新操作', FrameworkErrorEnum.SERVICE_FAULT);
    rres = JSON.parse(rres);
    if (get(rres, 'account') !== body.account) throw new ServiceError('该验证不是当前账号的验证码,拒绝操作', FrameworkErrorEnum.SERVICE_FAULT);
    const res = await this.service.updatePassword(body);
    return res;
  }
  @Post('/rp')
  @ApiOperation({ description: '修改密码' })
  async resetPassword(@Body() body: ResetPasswordDTO) {
    // 获取用户token中的登录信息
    const user_id = get(this.ctx, 'user._id');
    if (!user_id) throw new ServiceError('未找到用户信息', FrameworkErrorEnum.NOT_LOGIN);
    // 查询用户
    const user = await this.service.fetch(user_id);
    if (!user) throw new ServiceError('未找到用户', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 修改密码
    await this.service.updateOne(user_id, body);
    // 返回结果
    return 'ok';
  }

  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_user })
  async create(@Body() data: CreateDTO_user) {
    // 检查是否含有不允许字符
    const haveInvalid = RegisterCanNotIncludes.find(f => lowerCase(data.account).includes(f));
    if (haveInvalid) throw new ServiceError(`不允许包含特殊字符: ${RegisterCanNotIncludes.join('; ')}`);
    // 检查身份证号和登录账号是否唯一
    await this.userUtil.checkCardAndAccount(data);
    // 赋予 平台-个人账号角色
    data.role = ['63b64ac735df6c6862e0eba3','63b64b6335df6c6862e13f14','63b62d2a9bb09c9905e19063'];
    const result = await this.service.create(data);
    try {
      // // 生成各个项目之间的关系
      await this.service.toCreateModulesRelation(result);
      return result;
    } catch (error) {
      console.error(error);
      if (result) await this.delete(get(result, '_id'), false);
    }
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QueryVO_user })
  async query(@Query() filter: QueryDTO_user, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    for (let i of data) {
      i = await this.service.getUnitName(i);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_user })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FetchVO_user(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_user })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_user) {
    await this.userUtil.checkUpdateCardAndAccount(id, body);
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string, deleteRelation = true) {
    if (deleteRelation) {
      const data = await this.service.fetch(id);
      if (!data) throw new ServiceError('未找到数据', FrameworkErrorEnum.DATA_NOT_FOUND);
      // 删除各项目之间的关系
      await this.service.toDeleteModulesRelation(data);
    }

    await this.service.delete(id);
    return 'ok';
  }

  @Post('/refresh')
  async refreshToken() {
    const user = this.ctx.user;
    if (!user) throw new ServiceError('未找到登陆数据', FrameworkErrorEnum.DATA_NOT_FOUND);
    const _id = get(user, '_id');
    const newUser = await this.service.fetch(_id);
    let vo = new LoginVO(newUser);
    vo = JSON.parse(JSON.stringify(vo));
    // 找各个项目的关系
    await this.service.getModulesRelation(vo);
    // 判断可进入的模块
    await this.service.whitchModuleCanUse(vo);
    const token = await this.jwtService.sign(vo, this.jwtSecret, { expiresIn: this.jwtExpiresIn });
    return token;
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
