import { Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { BaseService, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { User } from '../entity/user.entity';
import { EmailResetPasswordDTO, LoginDTO, UpdateDTO_user } from '../interface/user.interface';
import isEqual = require('lodash/isEqual');
import get = require('lodash/get');
import { Types } from 'mongoose';
import { Unit } from '../entity/unit.entity';
import { Module } from '../entity/module.entity';
import { UserLab } from '../entityLab/user.entity';
import { UserProject } from '../entityProject/user.entity';

const ObjectId = Types.ObjectId;
type modelType = ReturnModelType<typeof User>;

@Provide()
export class UserService extends BaseService<modelType> {
  @InjectEntityModel(User)
  model: modelType;
  @InjectEntityModel(Module)
  moduleModel: ReturnModelType<typeof Module>;

  @InjectEntityModel(Unit)
  unitModel: ReturnModelType<typeof Unit>;

  @InjectEntityModel(UserLab)
  lModel: ReturnModelType<typeof UserLab>;

  @InjectEntityModel(UserProject)
  pModel: ReturnModelType<typeof UserProject>;

  async test() {
    const list = await this.model.find({ 'meta.createdAt': { $lte: '2023-02-01' } });
    console.log(list);
  }

  async checkUpdateCard(id: string, body: UpdateDTO_user) {
    if (!body.card) return;
    // 找: 非当前数据的 新身份证号的数据
    const number = await this.model.count({ _id: { $ne: id }, card: body.card });
    if (number > 0) throw new ServiceError('该身份证号已存在', FrameworkErrorEnum.BAD_BODY);
  }

  /**创建各项目与平台账号之间的关系 */
  async toCreateModulesRelation(data) {
    // 实验室创建关联
    await this.lModel.create({ user_id: data._id });
    // 项目征集创建关联
    await this.pModel.create({ user_id: data._id });
  }
  /**删除各项目与平台账号之间的关系 */
  async toDeleteModulesRelation(data) {
    // 实验室删除关联
    await this.lModel.deleteOne({ user_id: data._id });
    // 项目征集删除关联
    await this.pModel.deleteOne({ user_id: data._id });
  }
  /**查询各项目与平台账号之间的关系 */
  async getModulesRelation(data) {
    const uri = { user_id: data._id };
    const r1 = await this.lModel.findOne(uri);
    if (get(r1, '__v') === 0) {
      const lab_id = get(r1, '_doc.lab_id');
      const labUser = get(r1, 'user_id');
      if (!labUser) {
        // 实验室创建关联
        await this.lModel.create(uri);
        data.lab_id = undefined;
      } else data.lab_id = lab_id;
    }
    const r2 = await this.pModel.findOne(uri);
    if (get(r2, '__v') === 0) {
      const basic = get(r2, '_doc.basic');
      const projectUser = get(r2, 'user_id');
      if (!projectUser) {
        await this.pModel.create(uri);
        data.basic_id = undefined;
      } else data.basic_id = basic;
    }
  }
  /**
   * 判断可以使用哪些模块
   * @param data 登陆数据
   */
  async whitchModuleCanUse(data) {
    const modules = await this.moduleModel.find().lean();
    const entrances = {};
    for (const m of modules) {
      const { name, is_use, url } = m;
      if (is_use === 'N') entrances[name] = { is_use: false };
      else {
        let obj = {};
        if (name === 'project') {
          // 项目征集需要判断用户是否 关联 数据库的依托单位
          const unit = get(data, 'unit');
          if (ObjectId.isValid(unit)) obj = { is_use: true, url };
          else obj = { is_use: false };
        } else obj = { is_use: true, url };
        entrances[name] = obj;
      }
    }
    data.entrances = entrances;
  }

  async findUserToLogin(data: LoginDTO): Promise<object> {
    const { account, password } = data;
    const user = await this.model.findOne({ account }, '+password').lean();
    if (!user) throw new ServiceError('未找到用户信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    if (get(user, 'exam_status') !== '1') throw new ServiceError('当前用户没有通过审核,拒绝登录', FrameworkErrorEnum.SERVICE_FAULT);
    if (!isEqual(user.password.secret, password)) throw new ServiceError('密码错误', FrameworkErrorEnum.SERVICE_FAULT);
    return user;
  }

  async updatePassword(data: EmailResetPasswordDTO): Promise<string> {
    const { account, password } = data;
    await this.model.updateOne({ account }, { password });
    return 'ok';
  }
  /**统一添加unit_name字段 */
  async getUnitName(data) {
    const unit = get(data, 'unit');
    if (!unit) return data;
    if (ObjectId.isValid(unit)) {
      const udata = await this.unitModel.findOne({ _id: unit }, { unit_name: 1 }).lean();
      if (udata) data.unit_name = get(udata, 'unit_name');
    } else {
      data.unit_name = unit;
    }
    return data;
  }
}
